package com.framwork.page;

import com.util.Cacheable;
import com.util.CacheSizes;


public class StringList implements Cacheable {
    String[] elements;
    int capacity;
    int size;

    /**
     * Creates a new list of long values with a default capacity of 50.
     */
    public StringList() {
        this(50);
    }

    /**
     * Creates a new list of long values with a specified initial capacity.
     *
     * @param initialCapacity a capacity to initialize the list with.
     */
    public StringList(int initialCapacity) {
        size = 0;
        capacity = initialCapacity;
        elements = new String[capacity];
    }


    /**
     * Adds a new long value to the end of the list.
     */
    public void add(String value) {
        elements[size] = value;
        size++;
        if (size == capacity) {
            capacity = capacity * 2;
            String[] newElements = new String[capacity];
            for (int i = 0; i < size; i++) {
                newElements[i] = elements[i];
            }
            elements = newElements;
        }
    }

    /**
     * Returns the long value at the specified index. If the index is not
     * valid, an IndexOutOfBoundException will be thrown.
     *
     * @param index the index of the value to return.
     * @return the value at the specified index.
     */
    public String get(int index) {
        if (index < 0 || index >= size) {
            throw new IndexOutOfBoundsException("Index " + index + " not valid.");
        }
        return elements[index];
    }

    /**
     * Returns the number of elements in the list.
     *
     * @return the number of elements in the list.
     */
    public int size() {
        return size;
    }

    /**
     * Returns a new array containing the list elements.
     *
     * @return an array of the list elements.
     */
    public String[] toArray() {
        int size = this.size;
        String[] newElements = new String[size];
        for (int i = 0; i < size; i++) {
            newElements[i] = elements[i];
        }
        return newElements;
    }

    /**
     *  toStringSql
     * @return
     */
    public String toStringSql() {
        return toStringSql(true);
    }

    /**
     *  toStringSql
     * @param split
     * @return
     */

    public String toStringSql(boolean split) {
        StringBuffer buf = new StringBuffer();
        for (int i = 0; i < this.size; i++) {
            if (!split) buf.append("'");
            buf.append(elements[i]);
           if (!split) buf.append("'");
            buf.append(",");
        }
        //ɾһָ
        if(buf.lastIndexOf(",")!=-1)
        buf.deleteCharAt(buf.lastIndexOf(","));
        return buf.toString();
    }


    public String toString() {
        StringBuffer buf = new StringBuffer();
        for (int i = 0; i < this.size; i++) {
            buf.append(elements[i]).append(",");
        }

        //ɾһָ
        buf.deleteCharAt(buf.lastIndexOf(","));

        return buf.toString();
    }

    public long getSize() {
        return CacheSizes.sizeOfObject() + CacheSizes.sizeOfInt() * 2 + CacheSizes.sizeOfLong() * size;
    }
}
