package com.framwork.util;


import java.lang.reflect.*;

import java.security.spec.PKCS8EncodedKeySpec;
import java.security.spec.X509EncodedKeySpec;
import java.security.*;
import java.io.*;
import java.net.InetAddress;
import javax.crypto.Cipher;


public class RSATool {

    public RSATool() {
    }


    public static byte[] sign(byte[] keyInByte, byte[] source) {
        try {
            PKCS8EncodedKeySpec priv_spec = new PKCS8EncodedKeySpec(keyInByte);
            KeyFactory mykeyFactory = KeyFactory.getInstance("RSA");
            PrivateKey privKey = mykeyFactory.generatePrivate(priv_spec);
            Signature sig = Signature.getInstance("SHA1withRSA");
            sig.initSign(privKey);
            sig.update(source);
            return sig.sign();
        } catch (Exception e) {
            return null;
        }
    }


    public static boolean verify(byte[] keyInByte, byte[] source, byte[] sign) {
        try {
            KeyFactory mykeyFactory = KeyFactory.getInstance("RSA");
            Signature sig = Signature.getInstance("SHA1withRSA");
            X509EncodedKeySpec pub_spec = new X509EncodedKeySpec(keyInByte);
            PublicKey pubKey = mykeyFactory.generatePublic(pub_spec);
            sig.initVerify(pubKey);
            sig.update(source);
            return sig.verify(sign);
        } catch (Exception e) {
            return false;
        }
    }


    public static Object[] giveRSAKeyPairInByte() {
        KeyPair newKeyPair = creatmyKey();
        if (newKeyPair == null) return null;
        Object[] re = new Object[2];
        if (newKeyPair != null) {
            PrivateKey priv = newKeyPair.getPrivate();
            byte[] b_priv = priv.getEncoded();
            PublicKey pub = newKeyPair.getPublic();
            byte[] b_pub = pub.getEncoded();
            re[0] = b_priv;
            re[1] = b_pub;
            return re;
        }
        return null;
    }


    public static KeyPair creatmyKey() {
        KeyPair myPair;
        long mySeed;
        mySeed = System.currentTimeMillis();
        try {
            KeyPairGenerator keyGen = KeyPairGenerator.getInstance("RSA");
            SecureRandom random = SecureRandom.getInstance("SHA1PRNG", "SUN");
            random.setSeed(mySeed);
            keyGen.initialize(1024, random);
            myPair = keyGen.generateKeyPair();
        } catch (Exception e1) {
            return null;
        }
        return myPair;
    }


    public static byte[] encryptByRSA(byte[] pubKeyInByte, byte[] data) {
        try {
            KeyFactory mykeyFactory = KeyFactory.getInstance("RSA");
            X509EncodedKeySpec pub_spec = new X509EncodedKeySpec(pubKeyInByte);
            PublicKey pubKey = mykeyFactory.generatePublic(pub_spec);
            Cipher cipher = Cipher.getInstance("RSA/ECB/PKCS1Padding");
            cipher.init(Cipher.ENCRYPT_MODE, pubKey);
            return cipher.doFinal(data);
        } catch (Exception e) {
            return null;
        }
    }


    public static byte[] decryptByRSA(byte[] privKeyInByte, byte[] data) {
        try {
            PKCS8EncodedKeySpec priv_spec = new PKCS8EncodedKeySpec(privKeyInByte);
            KeyFactory mykeyFactory = KeyFactory.getInstance("RSA");
            PrivateKey privKey = mykeyFactory.generatePrivate(priv_spec);
            Cipher cipher = Cipher.getInstance("RSA/ECB/PKCS1Padding");
            cipher.init(Cipher.DECRYPT_MODE, privKey);
            return cipher.doFinal(data);
        } catch (Exception e) {
            return null;
        }

    }


    // ļbyte
    public static byte[] readFileData(String filename) {

        File file = new File(filename);
        long len = file.length();
        byte data[] = new byte[(int) len];
        FileInputStream fin = null;
        try {
            fin = new FileInputStream(file);
            int r = fin.read(data);
            if (r != len)
                throw new IOException("Only read " + r + " of " + len + " for " + file);
            fin.close();

        } catch (FileNotFoundException e) {
            e.printStackTrace();
        } catch (IOException ioe) {
            ioe.printStackTrace();
        }

        return data;
    }

    // byteдļ
    public static void writeFileData(String filename, byte data[]) {

        try {
            FileOutputStream fout = new FileOutputStream(filename);
            fout.write(data);
            fout.close();
        } catch (IOException e) {
            e.printStackTrace();
        }

    }

    public static byte[] MdigestSHA(byte[] source) {
        byte[] nullreturn = {0};
        try {
            MessageDigest thisMD = MessageDigest.getInstance("SHA");
            byte[] digest = thisMD.digest(source);
            return digest;
        } catch (Exception e) {
            return null;
        }
    }


    /**
     * return  sn from  number
     */
    public static String getSnOfNumber(String str) {

        if (str.equals("0"))
            return "CFCD208495D565EF66E7DFF9F98764DA";
        else if (str.equals("1"))
            return "C4CA4238A0B923820DCC509A6F75849B";
        else if (str.equals("2"))
            return "C81E728D9D4C2F636F067F89CC14862C";
        else if (str.equals("3"))
            return "ECCBC87E4B5CE2FE28308FD9F2A7BAF3";
        else if (str.equals("4"))
            return "A87FF679A2F3E71D9181A67B7542122C";
        else if (str.equals("5"))
            return "E4DA3B7FBBCE2345D7772B0674A318D5";
        else if (str.equals("6"))
            return "1679091C5A880FAF6FB5E6087EB1B2DC";
        else if (str.equals("7"))
            return "8F14E45FCEEA167A5A36DEDD4BEA2543";
        else if (str.equals("8"))
            return "C9F0F895FB98AB9159F51FD0297E236D";
        else if (str.equals("9"))
            return "45C48CCE2E2D7FBDEA1AFC51C7C6AD26";
        else
            return "";
    }


    public static String getMD5ofStr(String s) {
        return (new MD5()).getMD5ofStr(s);
    }


    public static void print(String s1) {
        String sn = "";
        char[] c_sn = s1.toCharArray();
        for (int j = 0; j < c_sn.length; j++) {
            sn += getSnOfNumber(c_sn[j] + "");
        }
        sn = getMD5ofStr(getDefaultMAC()) + sn;
        System.out.println("sn=" + sn);
        System.out.println("sn.length=" + sn.length());
    }


    //ȡIPַ
    public static String getIP() {
        try {
            InetAddress net = InetAddress.getLocalHost();
            String strIP = net.getHostAddress();
            return strIP;
        } catch (Exception e) {
            System.out.println(e.getMessage());
            return null;
        }
    }


    //ȡMACַ
    public static String getMAC() {

        String mac = "";

        try {

            Process process = Runtime.getRuntime().exec("nbtstat -A " + getIP());
            InputStreamReader ir = new InputStreamReader(process.getInputStream());
            LineNumberReader input = new LineNumberReader(ir);
            String line;
            while ((line = input.readLine()) != null) {
                if (line.indexOf("MAC Address") >= 0) {
                    int intStart;
                    int intEnd;
                    intStart = line.indexOf("=") + 2;
                    intEnd = intStart + 17;
                    mac = line.substring(intStart, intEnd);

                    return mac;
                }
            }
            return null;
        } catch (IOException e) {
            System.out.println(e.getMessage());
            return null;
        }
    }


    //ȡMACַ
    private static String getDefaultMAC() {
        String mac = getMAC(1);
        if (mac == null) {
            System.out.println("[" + new java.util.Date() + "] ȡϢʧܣ޷ϵͳ");
            System.exit(0);
        }

        return mac;

    }

    //ȡMACַ
    public static String getMAC(int num) {
        String mac = null;
        try {
            Process process = Runtime.getRuntime().exec("ipconfig -all ");
            InputStreamReader ir = new InputStreamReader(process.getInputStream());
            LineNumberReader input = new LineNumberReader(ir);
            String line;

            while ((line = input.readLine()) != null) {
                if (line.indexOf("Physical Address") >= 0) {
                    int intStart;
                    int intEnd;
                    intStart = line.indexOf(":") + 2;
                    intEnd = intStart + 17;
                    mac = line.substring(intStart, intEnd).trim();
                    if (num-- == 0) break;
                }
            }

            ir.close();
            input.close();

        } catch (IOException e) {
            System.out.println(e.getMessage());
        } finally {
            return mac;
        }
    }

    public void main_rsa(String args[]) {

        try {

            Object[] v = giveRSAKeyPairInByte();
            byte[] source = Digest.MdigestSHA("haha");
            byte[] sign = sign((byte[]) v[0], source);
            boolean yes = verify((byte[]) v[1], source, sign);
            if (yes) System.out.print("verify\n");

        } catch (Exception e) {
            e.printStackTrace();
        }


    }

    public void main_md5(String args[]) {


        MD5 m = new MD5();
        if (Array.getLength(args) == 0) {   //ûвִб׼Test Suite

            System.out.println("MD5 DaoTest suite:");
            System.out.println("MD5(\"\"):" + m.getMD5ofStr(""));
            System.out.println("MD5(\"a\"):" + m.getMD5ofStr("a"));
            System.out.println("MD5(\"abc\"):" + m.getMD5ofStr("abc"));
            System.out.println("MD5(\"message digest\"):" + m.getMD5ofStr("message digest"));
            System.out.println("MD5(\"abcdefghijklmnopqrstuvwxyz\"):" +
                    m.getMD5ofStr("abcdefghijklmnopqrstuvwxyz"));
            System.out.println("MD5(\"ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789\"):" +
                    m.getMD5ofStr("ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789"));
        } else
            System.out.println("MD5(" + args[0] + ")=" + m.getMD5ofStr(args[0]));


    }


    protected byte[] getDefaultFileData(String ext) {
        Class cls = getClass();
        java.io.ByteArrayOutputStream fos = new java.io.ByteArrayOutputStream();
        InputStream inputstream = null;
        inputstream = cls.getResourceAsStream(ext);
        java.io.BufferedOutputStream bos = new java.io.BufferedOutputStream(fos);
        byte[] buffer = new byte[1024];
        int i;
        try {
            while ((i = inputstream.read(buffer)) != -1) {
                bos.write(buffer, 0, i);
            }
            inputstream.close();
            bos.flush();
            bos.close();

        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            return fos.toByteArray();
        }
    }


    protected static byte[] readFileData(String s, String ext) {

        java.io.ByteArrayOutputStream fos = new java.io.ByteArrayOutputStream();
        InputStream inputstream = null;

        try {
            inputstream = new java.io.FileInputStream(new File(s));
            java.io.BufferedOutputStream bos = new java.io.BufferedOutputStream(fos);
            byte[] buffer = new byte[1];
            int i;
            while ((i = inputstream.read(buffer)) != -1) {
                bos.write(buffer, 0, i);
            }
            inputstream.close();
            bos.flush();
            bos.close();

        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            return fos.toByteArray();
        }
    }

    // ļbyte
    private static byte[] readFileData(String f, String f2, String f3, String f4) {

        File file = new File(f);
        File file2 = new File(f2);
        File file3 = new File(f3);
        File file4 = new File(f4);

        long len = file.length() + file2.length() + file3.length() + file4.length();

        byte data[] = new byte[(int) len];
        FileInputStream fin = null;
        FileInputStream fin2 = null;
        FileInputStream fin3 = null;
        FileInputStream fin4 = null;

        try {
            fin = new FileInputStream(file);
            fin2 = new FileInputStream(file2);
            fin3 = new FileInputStream(file3);
            fin4 = new FileInputStream(file4);

            int r = fin.read(data, 0, (int) file.length());
            int r2 = fin2.read(data, (int) file.length(), (int) file2.length());
            int r3 = fin3.read(data, (int) (file.length() + file2.length()), (int) file3.length());
            int r4 = fin4.read(data, (int) (file.length() + file2.length() + file3.length()), (int) file4.length());

            if (r + r2 + r3 + r4 != len)
                throw new IOException("Only read " + (r + r2 + r3 + r4) + " of " + len + " for readFileData() ");

            fin.close();
            fin2.close();
            fin3.close();
            fin4.close();

        } catch (FileNotFoundException e) {
            e.printStackTrace();
        } catch (IOException ioe) {
            ioe.printStackTrace();
        }

        return data;
    }


    /**
     *
     */
    public static void main(String[] args) {


        if (false) {

            //Կļ
            Object[] v = giveRSAKeyPairInByte();
            writeFileData("private.key", (byte[]) v[0]);
            writeFileData("WEB-INF/classes/public.key", (byte[]) v[1]);

        }

        byte[] source = Digest.MdigestSHA(readFileData("WEB-INF/classes/com/sx/common/SysCheck.class",
                "WEB-INF/classes/com/sx/common/CAuth.class",
                "WEB-INF/classes/com/sx/common/DbConnect.class",
                "WEB-INF/classes/com/sx/common/AppClass.class"));
        //ǩļ
        writeFileData("WEB-INF/classes/com/sx/common/SysCheck.key", sign(readFileData("private.key"), source));
        //˶ǩļ
        boolean yes = verify(readFileData("WEB-INF/classes/public.key"), source, readFileData("WEB-INF/classes/com/sx/common/SysCheck.key"));

        if (!yes)
            System.out.print("verify error \n");
        else
            System.out.println("verify successful");

        //к

        if (args==null||args.length == 0)
            print("200412305");

         else
             print(args[0]);
    }

}

class Digest {


    /**
     * ַSHAժҪbyte[]ʽ
     **/
    public static byte[] MdigestSHA(String source) {
        byte[] nullreturn = {0};
        try {
            MessageDigest thisMD = MessageDigest.getInstance("SHA");
            byte[] digest = thisMD.digest(source.getBytes("UTF8"));
            return digest;
        } catch (Exception e) {
            return null;
        }
    }

    /**
     * byte[]SHAժҪbyte[]ʽ
     **/
    public static byte[] MdigestSHA(byte[] source) {
        byte[] nullreturn = {0};
        try {
            MessageDigest thisMD = MessageDigest.getInstance("SHA");
            byte[] digest = thisMD.digest(source);
            return digest;
        } catch (Exception e) {
            return null;
        }
    }

    /**
     * ַMD5ժҪbyte[]ʽ
     **/
    public static byte[] MdigestMD5(String source) {
        byte[] nullreturn = {0};
        try {
            MessageDigest thisMD = MessageDigest.getInstance("MD5");
            byte[] digest = thisMD.digest(source.getBytes("UTF8"));
            return digest;
        } catch (Exception e) {
            return null;
        }
    }

    /**
     * byte[]MD5ժҪbyte[]ʽ
     **/
    public static byte[] MdigestMD5(byte[] source) {
        byte[] nullreturn = {0};
        try {
            MessageDigest thisMD = MessageDigest.getInstance("MD5");
            byte[] digest = thisMD.digest(source);
            return digest;
        } catch (Exception e) {
            return null;
        }
    }
}


/*************************************************
 md5 ʵRSA Data Security, Inc.ύIETF
 RFC1321еMD5 message-digest 㷨
 *************************************************/

class MD5 {
    /* ЩS11-S44ʵһ4*4ľԭʼCʵ#define ʵֵģ
    ʵֳΪstatic finalǱʾֻͬһ̿ռڵĶ
    Instance乲*/
    static final int S11 = 7;
    static final int S12 = 12;
    static final int S13 = 17;
    static final int S14 = 22;

    static final int S21 = 5;
    static final int S22 = 9;
    static final int S23 = 14;
    static final int S24 = 20;

    static final int S31 = 4;
    static final int S32 = 11;
    static final int S33 = 16;
    static final int S34 = 23;

    static final int S41 = 6;
    static final int S42 = 10;
    static final int S43 = 15;
    static final int S44 = 21;

    static final byte[] PADDING = {-128, 0, 0, 0, 0, 0, 0, 0, 0,
                                   0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
                                   0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
                                   0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
    /* ԱMD5õ3ݣԭʼCʵ
       嵽MD5_CTXṹ

     */
    private long[] state = new long[4];  // state (ABCD)
    private long[] count = new long[2];  // number of bits, modulo 2^64 (lsb first)
    private byte[] buffer = new byte[64]; // input buffer

    /* digestHexStrMD5ΨһһԱһμ
     16ASCIIʾ.
    */
    public String digestHexStr;

    /* digest,һμ2ڲʾʾ128bitMD5ֵ.
    */
    private byte[] digest = new byte[16];

    /*
      getMD5ofStrMD5ҪĹڲҪMD5任ַ
      صǱ任ĽǴӹԱdigestHexStrȡõģ
    */
    public String getMD5ofStr(String inbuf) {
        md5Init();
        md5Update(inbuf.getBytes(), inbuf.length());
        md5Final();
        digestHexStr = "";
        for (int i = 0; i < 16; i++) {
            digestHexStr += byteHEX(digest[i]);
        }
        return digestHexStr;

    }

    // MD5ı׼캯JavaBeanҪһpublicĲûвĹ캯
    public MD5() {
        md5Init();

        return;
    }


    /* md5Initһʼʼıװ׼Ļ */
    private void md5Init() {
        count[0] = 0L;
        count[1] = 0L;
        ///* Load magic initialization constants.

        state[0] = 0x67452301L;
        state[1] = 0xefcdab89L;
        state[2] = 0x98badcfeL;
        state[3] = 0x10325476L;

        return;
    }
    /* F, G, H ,I 4MD5ԭʼMD5CʵУ
    򵥵λ㣬ܳЧʵĿǰʵֳ˺꣬javaУǰ
   ʵֳprivateֱԭCеġ */

    private long F(long x, long y, long z) {
        return (x & y) | ((~x) & z);

    }

    private long G(long x, long y, long z) {
        return (x & z) | (y & (~z));

    }

    private long H(long x, long y, long z) {
        return x ^ y ^ z;
    }

    private long I(long x, long y, long z) {
        return y ^ (x | (~z));
    }

/*
    FF,GG,HHIIF,G,H,Iнһ任
    FF, GG, HH, and II transformations for rounds 1, 2, 3, and 4.
    Rotation is separate from addition to prevent recomputation.
 */

    private long FF(long a, long b, long c, long d, long x, long s,
                    long ac) {
        a += F(b, c, d) + x + ac;
        a = ((int) a << s) | ((int) a >>> (32 - s));
        a += b;
        return a;
    }

    private long GG(long a, long b, long c, long d, long x, long s,
                    long ac) {
        a += G(b, c, d) + x + ac;
        a = ((int) a << s) | ((int) a >>> (32 - s));
        a += b;
        return a;
    }

    private long HH(long a, long b, long c, long d, long x, long s,
                    long ac) {
        a += H(b, c, d) + x + ac;
        a = ((int) a << s) | ((int) a >>> (32 - s));
        a += b;
        return a;
    }

    private long II(long a, long b, long c, long d, long x, long s,
                    long ac) {
        a += I(b, c, d) + x + ac;
        a = ((int) a << s) | ((int) a >>> (32 - s));
        a += b;
        return a;
    }

    /*
     md5UpdateMD5̣inbufҪ任ֽڴinputlenǳȣ
     getMD5ofStrã֮ǰҪmd5init˰Ƴprivate
    */
    private void md5Update(byte[] inbuf, int inputLen) {

        int i, index, partLen;
        byte[] block = new byte[64];
        index = (int) (count[0] >>> 3) & 0x3F;
        // /* Update number of bits */
        if ((count[0] += (inputLen << 3)) < (inputLen << 3))
            count[1]++;
        count[1] += (inputLen >>> 29);

        partLen = 64 - index;

        // Transform as many times as possible.
        if (inputLen >= partLen) {
            md5Memcpy(buffer, inbuf, index, 0, partLen);
            md5Transform(buffer);

            for (i = partLen; i + 63 < inputLen; i += 64) {

                md5Memcpy(block, inbuf, 0, i, 64);
                md5Transform(block);
            }
            index = 0;
        } else
            i = 0;

        ///* Buffer remaining input */
        md5Memcpy(buffer, inbuf, index, i, inputLen - i);

    }

    /*
      md5Finalд
    */
    private void md5Final() {
        byte[] bits = new byte[8];
        int index, padLen;

        ///* Save number of bits */
        Encode(bits, count, 8);

        ///* Pad out to 56 mod 64.
        index = (int) (count[0] >>> 3) & 0x3f;
        padLen = (index < 56) ? (56 - index) : (120 - index);
        md5Update(PADDING, padLen);

        ///* Append length (before padding) */
        md5Update(bits, 8);

        ///* StoreImp state in digest */
        Encode(digest, state, 16);

    }

    /* md5MemcpyһڲʹõbyteĿ鿽inputinposʼlenȵ
    ֽڿoutputoutposλÿʼ
    */

    private void md5Memcpy(byte[] output, byte[] input, int outpos, int inpos, int len) {
        int i;

        for (i = 0; i < len; i++)
            output[outpos + i] = input[inpos + i];
    }

    /*
       md5TransformMD5ı任md5UpdateãblockǷֿԭʼֽ
    */
    private void md5Transform(byte block[]) {
        long a = state[0], b = state[1], c = state[2], d = state[3];
        long[] x = new long[16];

        Decode(x, block, 64);

        /* Round 1 */
        a = FF(a, b, c, d, x[0], S11, 0xd76aa478L); /* 1 */
        d = FF(d, a, b, c, x[1], S12, 0xe8c7b756L); /* 2 */
        c = FF(c, d, a, b, x[2], S13, 0x242070dbL); /* 3 */
        b = FF(b, c, d, a, x[3], S14, 0xc1bdceeeL); /* 4 */
        a = FF(a, b, c, d, x[4], S11, 0xf57c0fafL); /* 5 */
        d = FF(d, a, b, c, x[5], S12, 0x4787c62aL); /* 6 */
        c = FF(c, d, a, b, x[6], S13, 0xa8304613L); /* 7 */
        b = FF(b, c, d, a, x[7], S14, 0xfd469501L); /* 8 */
        a = FF(a, b, c, d, x[8], S11, 0x698098d8L); /* 9 */
        d = FF(d, a, b, c, x[9], S12, 0x8b44f7afL); /* 10 */
        c = FF(c, d, a, b, x[10], S13, 0xffff5bb1L); /* 11 */
        b = FF(b, c, d, a, x[11], S14, 0x895cd7beL); /* 12 */
        a = FF(a, b, c, d, x[12], S11, 0x6b901122L); /* 13 */
        d = FF(d, a, b, c, x[13], S12, 0xfd987193L); /* 14 */
        c = FF(c, d, a, b, x[14], S13, 0xa679438eL); /* 15 */
        b = FF(b, c, d, a, x[15], S14, 0x49b40821L); /* 16 */

        /* Round 2 */
        a = GG(a, b, c, d, x[1], S21, 0xf61e2562L); /* 17 */
        d = GG(d, a, b, c, x[6], S22, 0xc040b340L); /* 18 */
        c = GG(c, d, a, b, x[11], S23, 0x265e5a51L); /* 19 */
        b = GG(b, c, d, a, x[0], S24, 0xe9b6c7aaL); /* 20 */
        a = GG(a, b, c, d, x[5], S21, 0xd62f105dL); /* 21 */
        d = GG(d, a, b, c, x[10], S22, 0x2441453L); /* 22 */
        c = GG(c, d, a, b, x[15], S23, 0xd8a1e681L); /* 23 */
        b = GG(b, c, d, a, x[4], S24, 0xe7d3fbc8L); /* 24 */
        a = GG(a, b, c, d, x[9], S21, 0x21e1cde6L); /* 25 */
        d = GG(d, a, b, c, x[14], S22, 0xc33707d6L); /* 26 */
        c = GG(c, d, a, b, x[3], S23, 0xf4d50d87L); /* 27 */
        b = GG(b, c, d, a, x[8], S24, 0x455a14edL); /* 28 */
        a = GG(a, b, c, d, x[13], S21, 0xa9e3e905L); /* 29 */
        d = GG(d, a, b, c, x[2], S22, 0xfcefa3f8L); /* 30 */
        c = GG(c, d, a, b, x[7], S23, 0x676f02d9L); /* 31 */
        b = GG(b, c, d, a, x[12], S24, 0x8d2a4c8aL); /* 32 */

        /* Round 3 */
        a = HH(a, b, c, d, x[5], S31, 0xfffa3942L); /* 33 */
        d = HH(d, a, b, c, x[8], S32, 0x8771f681L); /* 34 */
        c = HH(c, d, a, b, x[11], S33, 0x6d9d6122L); /* 35 */
        b = HH(b, c, d, a, x[14], S34, 0xfde5380cL); /* 36 */
        a = HH(a, b, c, d, x[1], S31, 0xa4beea44L); /* 37 */
        d = HH(d, a, b, c, x[4], S32, 0x4bdecfa9L); /* 38 */
        c = HH(c, d, a, b, x[7], S33, 0xf6bb4b60L); /* 39 */
        b = HH(b, c, d, a, x[10], S34, 0xbebfbc70L); /* 40 */
        a = HH(a, b, c, d, x[13], S31, 0x289b7ec6L); /* 41 */
        d = HH(d, a, b, c, x[0], S32, 0xeaa127faL); /* 42 */
        c = HH(c, d, a, b, x[3], S33, 0xd4ef3085L); /* 43 */
        b = HH(b, c, d, a, x[6], S34, 0x4881d05L); /* 44 */
        a = HH(a, b, c, d, x[9], S31, 0xd9d4d039L); /* 45 */
        d = HH(d, a, b, c, x[12], S32, 0xe6db99e5L); /* 46 */
        c = HH(c, d, a, b, x[15], S33, 0x1fa27cf8L); /* 47 */
        b = HH(b, c, d, a, x[2], S34, 0xc4ac5665L); /* 48 */

        /* Round 4 */
        a = II(a, b, c, d, x[0], S41, 0xf4292244L); /* 49 */
        d = II(d, a, b, c, x[7], S42, 0x432aff97L); /* 50 */
        c = II(c, d, a, b, x[14], S43, 0xab9423a7L); /* 51 */
        b = II(b, c, d, a, x[5], S44, 0xfc93a039L); /* 52 */
        a = II(a, b, c, d, x[12], S41, 0x655b59c3L); /* 53 */
        d = II(d, a, b, c, x[3], S42, 0x8f0ccc92L); /* 54 */
        c = II(c, d, a, b, x[10], S43, 0xffeff47dL); /* 55 */
        b = II(b, c, d, a, x[1], S44, 0x85845dd1L); /* 56 */
        a = II(a, b, c, d, x[8], S41, 0x6fa87e4fL); /* 57 */
        d = II(d, a, b, c, x[15], S42, 0xfe2ce6e0L); /* 58 */
        c = II(c, d, a, b, x[6], S43, 0xa3014314L); /* 59 */
        b = II(b, c, d, a, x[13], S44, 0x4e0811a1L); /* 60 */
        a = II(a, b, c, d, x[4], S41, 0xf7537e82L); /* 61 */
        d = II(d, a, b, c, x[11], S42, 0xbd3af235L); /* 62 */
        c = II(c, d, a, b, x[2], S43, 0x2ad7d2bbL); /* 63 */
        b = II(b, c, d, a, x[9], S44, 0xeb86d391L); /* 64 */

        state[0] += a;
        state[1] += b;
        state[2] += c;
        state[3] += d;

    }

    /*Encodelong鰴˳byte飬Ϊjavalong64bitģ
      ֻ32bitӦԭʼCʵֵ;
    */
    private void Encode(byte[] output, long[] input, int len) {
        int i, j;

        for (i = 0, j = 0; j < len; i++, j += 4) {
            output[j] = (byte) (input[i] & 0xffL);
            output[j + 1] = (byte) ((input[i] >>> 8) & 0xffL);
            output[j + 2] = (byte) ((input[i] >>> 16) & 0xffL);
            output[j + 3] = (byte) ((input[i] >>> 24) & 0xffL);
        }
    }

    /*Decodebyte鰴˳ϳɳlong飬Ϊjavalong64bitģ
      ֻϳɵ32bit32bit㣬ӦԭʼCʵֵ;
    */
    private void Decode(long[] output, byte[] input, int len) {
        int i, j;


        for (i = 0, j = 0; j < len; i++, j += 4)
            output[i] = b2iu(input[j]) | (b2iu(input[j + 1]) << 8) | (b2iu(input[j + 2]) << 16) | (b2iu(input[j + 3]) << 24);

        return;
    }

    /*
      b2iuдһbyteղŵԭģλΪjavaûunsigned
    */
    public static long b2iu(byte b) {
        return b < 0 ? b & 0x7F + 128 : b;
    }

    /*byteHEX()һbyte͵תʮƵASCIIʾ
    ΪjavaеbytetoString޷ʵһ㣬ûCе
      sprintf(outbuf,"%02X",ib)
    */
    public static String byteHEX(byte ib) {
        char[] Digit = {'0', '1', '2', '3', '4', '5', '6', '7', '8', '9',
                        'A', 'B', 'C', 'D', 'E', 'F'};
        char[] ob = new char[2];
        ob[0] = Digit[(ib >>> 4) & 0X0F];
        ob[1] = Digit[ib & 0X0F];
        String s = new String(ob);
        return s;
    }
}