package com.framwork.web;

import com.framwork.aop.AopClient;
import com.framwork.model.TargetMetaDef;
import com.framwork.model.ComponentCacheVisitor;
import com.framwork.model.Visitable;
import com.framwork.util.Debug;

import java.lang.reflect.Proxy;

public class ProxyInstanceVisitable implements Visitable {

    private final static String module = ProxyInstanceVisitable.class.getName();

    private AopClient aopClient;
    private TargetMetaDef targetMetaDef;

    public ProxyInstanceVisitable(TargetMetaDef targetMetaDef,  AopClient aopClient) {
        this.aopClient = aopClient;
        this.targetMetaDef = targetMetaDef;
    }

    public Object accept(ComponentCacheVisitor visitor) {
        return createDynamicProxy(visitor);
    }

    public Object createDynamicProxy(ComponentCacheVisitor cm) {
        Object dynamicProxy = null;
        try {

            WebInvocationHandler handler = createInvocationHandler(cm);
            ClassLoader classLoader = Thread.currentThread().getContextClassLoader();
            Class serviceClass = classLoader.loadClass(targetMetaDef.getClassName());
            dynamicProxy =
                    Proxy.newProxyInstance(
                            classLoader,
                            new Class[]{serviceClass} ,
                            handler);
        } catch (Exception ex) {
            Debug.logError(" Proxy.newProxyInstance error:" + ex, module);

        } catch (Throwable ex) {
            Debug.logError(" Proxy.newProxyInstance error:" + ex, module);

        }
        return dynamicProxy;
    }

    private WebInvocationHandler createInvocationHandler(ComponentCacheVisitor cm) {

        WebInvocationHandler handler = new WebInvocationHandler(targetMetaDef, aopClient);
        handler.setVisitor(cm);
        return handler;
    }

}
